var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-08-28\">\n      <meta name=\"topicid\" content=\"GUID-24998B9E-2FF6-4161-8D31-D72AC1BFC54F\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Custom Attributes Version Number Handling</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-24998B9E-2FF6-4161-8D31-D72AC1BFC54F\"></a><div class=\"head\">\n            <h1>Custom Attributes Version Number Handling </h1>\n         </div>\n         <p><a name=\"GUID-69B5EDE7-96A8-4384-B7F4-C6246A10582F\"></a> Handling of version numbers when loading scripted custom attributes from scene files\n            was added in <em class=\"strong\">3ds Max 2012</em>. \n         </p>\n         <div class=\"section\"><a name=\"SECTION_066DA403BAD14630AA472BA3D8F48D02\"></a> \n            <h2><a name=\"GUID-DED10D5E-7508-401A-AAF2-681BA27A7C03\"></a>Technical Overview\n            </h2> \n            <p><a name=\"P_C7A965CC94A14CE899D1E11A2B928B19\"></a>The definition of scripted custom attributes is stored in the scene file in the \"ScriptedCustAttribDefs\"\n               stream, which is read in ILoadImp::LoadDllDir. This is called early in the file load\n               process, immediately before reading in the list of used classes, since the used classes\n               will include the scripted custom attribute classes. \n            </p> \n            <p><a name=\"P_1F0F872D023B485AA432E728A548255F\"></a>In <em class=\"strong\">3ds Max</em> versions <em class=\"strong\">prior to 3ds Max 2012</em>, when reading the scripted custom attributes definitions, if a scripted custom attributes\n               definition did not exactly match an existing definition, the loaded definition was\n               evaluated, which converted <em class=\"strong\">any existing instances</em> of that scripted custom attribute to the <em class=\"strong\">loaded definition</em>. \n            </p> \n            <p><a name=\"P_0918D29364814F24B6AAB3357C4B492F\"></a>We now want to evaluate the loaded definition only if it <em class=\"strong\">does not exactly match</em> an existing definition <em class=\"strong\">and </em>the loaded definition has a <em class=\"strong\">higher version number </em>than the existing version number. \n            </p> \n            <p><a name=\"P_E425FB6B72B94C419EC257229F000765\"></a>Since earlier versions of <em class=\"strong\">3ds Max</em> were always updating the scripted custom attributes definitions to the loaded definitions,\n               they never had to migrate the instances created while loading the scene file to the\n               existing definition, but starting with <em class=\"strong\">3ds Max 2012</em>, this has to be done. \n            </p> \n            <div><a name=\"GUID-49BDB463-071E-49DA-A4EF-62A8EEAED4E4\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span> \n                  <p><a name=\"P_E39CB57C503D46F68F47789FAA0B4A99\"></a>Note that the internal logic for migrating was already present, since the same process\n                     is performed for scripted plugins. \n                  </p> \n               </div>\n            </div> \n            <p><a name=\"P_D40B724AB2A24AA288EE26DF7D05EDDD\"></a>However, to do this migration requires data loaded from the \"Config\" stream, which\n               hasn\'t been read yet. \n            </p> \n         </div><a name=\"TABLE_0DEBA6FA9F174A61A1FEC67DA70A481A\"></a><div class=\"table-ruled\">\n            <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n               <colgroup>\n                  <col>\n                  <col>\n               </colgroup>\n               <tr class=\"ruled-heading\">\n                  <th class=\"table-heading\">Loading order in versions prior to 3ds Max 2012: </th>\n                  <th class=\"table-heading\">Loading order since 3ds Max 2012: </th>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> <a name=\"OL_53D4265B023B438A82D416097048FAE2\"></a><ol type=\"1\" start=\"1\">\n                        <li> Load ScriptedCustAttribDefs, updating current instances to loaded defs </li>\n                        <li> Load reference hierarchy </li>\n                        <li> Run postload callbacks </li>\n                        <li> Load config data, which includes the maxscript data containing the pb2 definitions\n                           for scripted plugins \n                        </li>\n                        <li> Run postload callbacks, which will migrate scripted plugin paramblocks to the current\n                           scripted plugin definitions \n                        </li>\n                     </ol> <a name=\"P_D6AED0E6AA4347B58860C48CDAD9AFFC\"></a><p class=\"table-body\">Note that in some cases, <em class=\"strong\">scene evaluations</em> can occur at <em class=\"strong\">step 3</em>. If those evaluations include the evaluation of the scripted plugin,<em class=\"strong\"> errors could occur</em> because at that point their param blocks may not correspond to the scripted plugin\n                        definitions. \n                     </p> \n                  </td>\n                  <td class=\"table-body\"> <a name=\"OL_85480D90131A404388CB3BBE5FFF89BD\"></a><ol type=\"1\" start=\"1\">\n                        <li> Load ScriptedCustAttribDefs, updating current instances to loaded defs <em class=\"strong\">if loaded def are newer version</em>. \n                        </li>\n                        <li> Load reference hierarchy </li>\n                        <li><em class=\"strong\"> Load part of the config data</em> - the MAXScript data containing the pb2 definitions for scripted plugins \n                        </li>\n                        <li> Run postload callbacks, which will migrate scripted plugin paramblocks to the current\n                           scripted plugin definitions \n                        </li>\n                        <li><em class=\"strong\"> Load remaining config data</em> - excludes the MAXScript data containing the pb2 definitions for scripted plugins\n                           \n                        </li>\n                        <li> Run postload callbacks. </li>\n                     </ol> <a name=\"P_0155E2D49DF740AEB0D5FD9936B70D6B\"></a><p class=\"table-body\">Note that if you <em class=\"strong\">evaluate</em> a scripted custom attribute definition (or a scripted plugin definition), the evaluated\n                        definition is <em class=\"strong\">always used </em>regardless of the version number of a previously existing definition. \n                     </p> \n                  </td>\n               </tr>\n            </table>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";