var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-08-28\">\n      <meta name=\"topicid\" content=\"GUID-382E7583-DD2A-4DF5-B568-71502DF95ED9\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"indexterm\" content=\"variables\">\n      <meta name=\"indexterm\" content=\"scope of variables\">\n      <meta name=\"indexterm\" content=\"local\">\n      <meta name=\"indexterm\" content=\"global\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Scope of Variables</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-382E7583-DD2A-4DF5-B568-71502DF95ED9\"></a><div class=\"head\">\n            <h1>Scope of Variables </h1>\n         </div>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868CC\"></a>    Variables have an attribute called <em class=\"mild\">scope</em> that determines where a variable can be accessed in MAXScript code. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868D1\"></a>MAXScript has two kinds of variable scope, global and local. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868D2\"></a> <em class=\"mild\">Global variables</em> are visible in all running MAXScript code and hold their values until you exit <em class=\"strong\">3ds Max</em>. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868D3\"></a> <em class=\"mild\">Local variables</em> are directly visible to code only in the lexical scope currently in effect, and hold\n            their values only as long as the current scope is active. After the scope is no longer\n            active, you can no longer access the contents of a variable local to that scope. This\n            is similar to how most modern programming languages implement variables. The conditions\n            under which MAXScript creates a new scope are described <span class=\"char_link\"><a href=\"GUID-382E7583-DD2A-4DF5-B568-71502DF95ED9.htm#WS3ED54CBA79FF2E3D1E593B6612B78359E8F-7668\">further on this page</a></span>. \n         </p>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E868D5\"></a> \n            <h2><a name=\"GUID-86D720E0-EC46-4324-A929-FFEC81417D5F\"></a>Explicit Variable Declaration\n            </h2> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868D6\"></a>MAXScript provides language constructs used to <em class=\"strong\">explicitly</em> declare and initialize both global and local variables. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868D7\"></a>The syntax for the variable declaration (<span class=\"code\"> <em class=\"strong\">&lt;variable_decls&gt;</em> </span>) is as follows: \n            </p> \n            <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61AD\"></a><pre>( <em class=\"codeEmphasisStrong\">local</em> | <em class=\"codeEmphasisStrong\">global</em> ) &lt;decl&gt; { , &lt;decl&gt; }</pre></div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868D9\"></a>where <span class=\"code\"> <em class=\"strong\">&lt;decl&gt;</em> </span> is defined as: \n            </p> \n            <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61AC\"></a><pre>&lt;var_name&gt; [ = &lt;expr&gt; ] -- name and optional initial value</pre></div> <a name=\"WS73099CC142F48755-2231E4B3128F27532E868DB\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E868DC\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLES:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61AB\"></a><pre>\nglobal baz -- initialized to \'undefined\'\nglobal foo=10, initialized=false -- initialized globals\nlocal x = 1, -- continued on several lines\ny = 2,\nz = sqrt(123.7)\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868E5\"></a>As mentioned above, local variables hold their values as long as the scope is active.\n               In nested scopes, this is usually a very short period. It starts when a function or\n               loop or block expression runs and ends when it exits. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868E6\"></a>Each time the scope is entered at runtime, a new set of locals is created and then\n               retired when the scope exits. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868E8\"></a>It is a good practice to always declare variables explicitly. See <span class=\"char_link\"><a href=\"GUID-382E7583-DD2A-4DF5-B568-71502DF95ED9.htm#WS3ED54CBA79FF2E3D1E593B6612B78359E8F-7631\">further on this page</a></span> for reasons. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E868EA\"></a> \n            <h2><a name=\"GUID-BD7C2EAA-55C0-46BF-9776-31922F73D33D\"></a>Explicit Global Variable Access\n            </h2> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868EB\"></a>Since <em class=\"strong\">3ds Max 8</em>, MAXScript provides a way to explicitly access a variable name as global. If the\n               variable exists in the global pool of variables, it will be used. If the variable\n               name does not exist in the global scope, it will be created as explicitly global.\n               \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868EC\"></a>The syntax is double-colon \'::\' and is <span class=\"char_link\"><a href=\"GUID-3E792B8C-AF02-4178-9651-EDDB1341B68D.htm\">described in this topic</a></span>. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"WS3ED54CBA79FF2E3D1E593B6612B78359E8F-7675\"></a> \n            <h2><a name=\"GUID-52170AD9-FF5E-4994-88A9-3344C31C404F\"></a> <em class=\"strong\">Local Variables as \'Private Global\' variables</em> \n            </h2> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868EE\"></a>In certain situations, the top-level scope can remain active for extended periods,\n               so the locals declared at the top-level hold their values for that extended period.\n               \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868EF\"></a>In particular, top-level locals declared in scripted rollouts, utilities, plug-ins,\n               script controllers, and Macro Scripts hold their values for as long as the rollout,\n               utility, plug-in, or Macro Script exists. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868F0\"></a>Typically, they remain in existence until you redefine them, for example, when you\n               define a Macro Script and run it for the first time, the top-level scope is created\n               and remains active over any number of subsequent executions unless you redefine the\n               Macro Script. At this point, the existing top-level scope is retired and a fresh one\n               (along with its top-level locals) is created. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868F2\"></a>This lets you use top-level locals in these constructs as a kind of <em class=\"strong\">private global</em>; the values they hold remain in place over many executions, but only the code in\n               that construct can see the variable, so it does not conflict with actual global variables.\n               \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E868F3\"></a>Note that in scripted plug-ins, a separate copy of the top-level local scope is created\n               for <em class=\"mild\">each</em> instance of the plugin and this scope remains active while that instance remains\n               in existence up until the end of the current <em class=\"strong\">3ds Max</em> session. \n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E868F5\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E868F6\"></a><p class=\"table-body\"> <em class=\"strong\">HISTORY NOTE:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\">In <em class=\"strong\">3ds Max</em> R2.x, the optional initialization value was applied to global variables only when\n                        the declaration creates a new variable. For example, if you ran the following script\n                        twice, the value of x in the second line will be 10 for the first execution, and 20\n                        for the second execution. In <em class=\"strong\">3ds Max 3</em> and later, the optional initialization value is always applied to global variables.\n                        \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61AA\"></a><pre>global x = 10\nprint x\nx=20</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E868FD\"></a><p class=\"table-body\">If you want to make the global variable initialization conditional in <em class=\"strong\">3ds Max 3 and higher</em>, use a construct similar to: \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61A9\"></a><pre>global foo; if foo = = undefined do foo = 23</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"WS3ED54CBA79FF2E3D1E593B6612B78359E8F-7668\"></a> \n            <h2><a name=\"GUID-A17833D6-B224-42E6-8E3B-FECD67E9110E\"></a> <em class=\"strong\">Implicit Variable Declaration</em> \n            </h2> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86902\"></a>If you do not explicitly declare a variable and the variable name does not exist at\n               a higher scope, MAXScript will create the variable the first time you use it and initialize\n               it to hold the special value <span class=\"code\"> <em class=\"strong\">undefined</em> </span>. You are not required to explicitly declare a variable or initialize its value before\n               you can use it. Variable names not explicitly declared by one of the previous statements\n               are called implicitly declared variables. The scope of implicitly declared variables\n               is the MAXScript scope context that is currently in effect when the variable is first\n               used. The initial MAXScript scope context is global, and new scope contexts are opened\n               for the following: \n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86903\"></a><ul>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86905\"></a>Top-level open parentheses in a script file or in the Listener \n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86907\"></a>Start of a function body \n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86909\"></a>Start of a <span class=\"code\"> <em class=\"strong\">for </em> </span> loop body \n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E8690B\"></a>Start of a utility, rollout, right-click menu, Macro Script, or tool definition \n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E8690D\"></a>Start of a rollout, utility, right-click menu, Macro Script, or tool event handler\n                     \n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E8690F\"></a>Start of a <span class=\"code\"> <em class=\"strong\">when</em> </span> body \n                  </p> \n               </li>\n            </ul> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86910\"></a>Within these new scopes, newly referenced variables will be implicitly declared as\n               local variables. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86911\"></a>Scope contexts are nested with the scope of a variable explicitly or implicitly declared\n               at one scope context level extending to all scope contexts below that level. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86912\"></a>Take for example the following script: \n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86913\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"15.356711003627572%\">\n                     <col width=\"84.64328899637243%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86914\"></a><p class=\"table-body\"> <em class=\"strong\">LINE #</em> \n                        </p> \n                     </td>\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86915\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61A8\"></a><pre>\n1\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n</pre></div> \n                     </td>\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61A7\"></a><pre>\na=10 -- scope context: global\n( b=20 -- new scope context: level 1\nfor i = 1 to 5 do -- new scope context: level 2\n( j=random i a\nk=random i b\nprint (j*a+k*b)\n) -- end of scope context: level 2\na=a+b\n) -- end of scope context: level 1\nprint a -- back to global scope context\nprint k\n</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E8692F\"></a><p class=\"table-body\"> <em class=\"strong\">RESULT #1</em> \n                        </p> \n                     </td>\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86930\"></a><p class=\"table-body\"> <em class=\"strong\">COMMENTS</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61A6\"></a><pre>\n10\n170\n120\n290\n320\n360\n30\n30\n30\nundefined\nundefined\n</pre></div> \n                     </td>\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>\nresult of a=10\nprint from i=1\nprint from i=2\nprint from i=3\nprint from i=4\nprint from i=5\nreturn value of end of scope level 1\nresult of print a\nreturn value of print a\nresult of print k\nreturn value of k</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"P_E502A0A6852C4D07B1530DC58F9A1674\"></a><p class=\"table-body\"> <em class=\"strong\">RESULT #2</em> \n                        </p> \n                     </td>\n                     <td class=\"table-body\"> <a name=\"P_95DFF8106E0D4C4881148830B5BFDC9E\"></a><p class=\"table-body\"> <em class=\"strong\">COMMENTS</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61A4\"></a><pre>\n10\n170\n420\n410\n180\n160\n30\n30\n30\n5\n5\n</pre></div> \n                     </td>\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>\nresult of a=10\nresult of print from loop i=1\nresult of print from loop i=2\nresult of print from loop i=3\nresult of print from loop i=4\nresult of print from loop i=5\nreturn value at end of scope level 1\nresult of print a\nreturn value of print a\nresult of print k\nreturn value of k</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86965\"></a>In the above script, variable <span class=\"code\"> <em class=\"strong\">a</em> </span> is first used in the global scope context and its scope includes scope contexts global,\n               level 1, and level 2. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86966\"></a>Variable <span class=\"code\"> <em class=\"strong\">b</em> </span> is first used in scope context level 1, so it is implicitly declared as a local variable\n               and its scope includes scope contexts level 1 and 2. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86967\"></a>Variables <span class=\"code\"> <em class=\"strong\">i</em> </span> and <span class=\"code\"> <em class=\"strong\">j</em> </span> are first used in scope context level 2 and their scope is only scope context level\n               2. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86968\"></a>The scope of variable <span class=\"code\"> <em class=\"strong\">k</em> </span> varies depending on whether this script has been run before. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86969\"></a>The first time the script is run, variable <span class=\"code\"> <em class=\"strong\">k</em> </span> is first defined at line 5 and its scope is scope context level 2. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E8696A\"></a>In line 11, variable <span class=\"code\"> <em class=\"strong\">k</em> </span> is used again. Because the variable <span class=\"code\"> <em class=\"strong\">k</em> </span> defined in line 5 is no longer in scope, a new variable <span class=\"code\"> <em class=\"strong\">k</em> </span> is defined whose scope is global. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E8696B\"></a>The second time you run the script, at line 5 MAXScript detects that variable <span class=\"code\"> <em class=\"strong\">k</em> </span> already exists and uses it. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E8696C\"></a>So, the <em class=\"strong\">first time</em> you run the script, line 11 will print <span class=\"code\"> <em class=\"strong\">undefined</em> </span>, and the <em class=\"strong\">second time</em> you run the script, line 11 will print the last value calculated at line 5. \n            </p> \n            <div><a name=\"WS1A9193826455F5FF-56C0CD8012A8003217A-193C\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>The scope of the variable used as a <span class=\"code\"> <em class=\"strong\">for</em> </span> loop counter (variable <span class=\"code\"> <em class=\"strong\">i</em> </span> in the above script) is a special case. The <span class=\"code\"> <em class=\"strong\">for</em> </span> loop counter variable\'s scope is always the scope context created for the <span class=\"code\"> <em class=\"strong\">for</em> </span> loop. This is true even if the <span class=\"code\"> <em class=\"strong\">for</em> </span> loop counter variable\'s name has already been implicitly or explicitly declared at\n                  a higher scope context level. \n               </div>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"WS3ED54CBA79FF2E3D1E593B6612B78359E8F-7642\"></a> \n            <h2><a name=\"GUID-9D22F329-7907-4EFA-B0C7-EECA4A9B39FE\"></a> <em class=\"strong\">Lexical Scoping</em> \n            </h2> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86971\"></a>When you explicitly declare a local variable, you can reuse the same name as a variable\n               at a higher scope context level. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86972\"></a>If you do this, the newly declared local variable hides the outer variables with the\n               same name. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86973\"></a>Any reference to that variable name later in the local variable\'s scope refers to\n               the new local variable. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86974\"></a>At the end of the local variable\'s scope, the next outer variable becomes visible\n               again. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86975\"></a>This visibility scheme is called <em class=\"mild\">lexical scoping</em>. An example of lexical scoping is shown in the following script. \n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86976\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"48.45360824742268%\">\n                     <col width=\"51.54639175257733%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86977\"></a><p class=\"table-body\"># </p> \n                     </td>\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86978\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61A2\"></a><pre>\n1\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n15\n16\n17\n18\n19\n20\n21\n22\n23\n24\n25\n26\n</pre></div> \n                     </td>\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61A1\"></a><pre>\nglobal foo = 23, x = 20 -- explicit declaration of foo and x,\n-- scope is global\ny = 10 -- implicit declaration of y,\n-- scope is global\nformat \"context level global: foo= %\\n\" foo\nif x &gt; y then\n( -- top-level open parentheses\n-- new scope is created\nlocal baz = foo + 1 -- uses the global foo\nlocal foo = y - 1 -- declares 1st local foo, hides global foo\nformat \"context level 1: foo= %\\n\" foo\nb=box() -- b implicitly declared as local\nb.pos.x = foo -- uses 1st local foo\nif (foo &gt; 0) then\n(\nlocal a\nlocal foo = y - x -- a nested 2nd local foo, hides 1st local\nformat \" context level 2: foo= %\\n\" foo\na = sin foo -- uses 2nd local foo\nformat \"a= %\\n\" a\n) -- leave scope\nb.pos.y= foo - 1.5 -- back to 1st local foo\nformat \"context level 1: foo= %\\n\" foo\n) -- leave scope\n-- b, baz and foo no longer in scope\nformat \"context level global: foo= %\\n\" foo -- back to global foo\n</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E869B0\"></a><p class=\"table-body\">&nbsp; </p> \n                     </td>\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E869B1\"></a><p class=\"table-body\"> <em class=\"strong\">OUTPUT</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E869B2\"></a><p class=\"table-body\">&nbsp; </p> \n                     </td>\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-61A0\"></a><pre>\n20 -- result of line 1\n10 -- result of line 3\ncontext level global - foo= 23 -- output from line 5\nOK -- result of line 5\ncontext level 1: foo= 9 -- output from line 11\ncontext level 2: foo= -10 -- output from line 18\na= -0.173648 -- output from line 20\ncontext level 1: foo= -10 -- output from line 23\nOK -- result of if expression lines 6 to 24\ncontext level global: foo= 23 -- output from line 26\nOK -- result of line 26\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869C1\"></a>This might seem a strange way of over-using a single variable name, but these scoping\n               rules can be useful in large programs. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869C2\"></a>For example, you might add a new user-interface item and its handlers to a utility\n               script. By explicitly declaring the variables used in the handlers as local, you ensure\n               these variables are independent of any pre-existing variables with the same names\n               in the remainder of the script. \n            </p> \n            <p><a name=\"WS3ED54CBA79FF2E3D1E593B6612B78359E8F-7631\"></a>When writing scripts, it is <em class=\"strong\">good programming practice</em> to <em class=\"strong\">explicitly declare</em> your local and global variables. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869C4\"></a>Implicit declaration is provided as a <em class=\"strong\">short-hand</em>, typically used when working in the Listener interactively or developing short scripts.\n               \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869C5\"></a>When developing extended scripts, <em class=\"strong\">explicitly declaring</em> variables can <em class=\"strong\">reduce errors</em> and improve readability of the code. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869C6\"></a>It is also recommended that you declare as local all variables unless you really want\n               them to be global variables. There are several reasons for this practice: \n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E869C7\"></a><ul>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869C9\"></a>All the variable names used in the block or function will be together, which makes\n                     it easier to find the variable names and helps prevent using incorrect names in the\n                     script. \n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869CB\"></a>If more than one script is executed at the same time (for example, you are running\n                     a scripted utility and have a scripted controller in the scene), and both scripts\n                     use the same global variable name, the two scripts can interfere with one another.\n                     This can cause random failures of one or both scripts. \n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869CD\"></a>You are sure of using the correct variable in the event that a variable with the same\n                     name has already been declared at a higher scope, and you cannot inadvertently change\n                     the higher-level variable\'s value. \n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869CF\"></a>Values of variables explicitly defined as local are displayed in error call stack\n                     trace-backs. \n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869D1\"></a>In <em class=\"strong\">3ds Max</em> R2.5, if a <em class=\"strong\">3ds Max</em> class, a MAXScript method, or a MAXScript read-only global variable has the same\n                     name as an implicitly declared local variable, a MAXScript runtime error will be generated\n                     if an assignment to that variable name occurs. Because the variable name already existed\n                     with a global scope, an implicitly declared local variable is not created. As one\n                     or more new <em class=\"strong\">3ds Max</em> classes will be created by any third party plug-ins the user has loaded, you cannot\n                     tell ahead of time if a variable name is a global variable name. In <em class=\"strong\">3ds Max</em> R3, if the MAXScript compiler detects that the first reference to such a variable\n                     is an assignment, it will implicitly declare a local variable, rather than leaving\n                     it as a reference to a read-only system global. For example, executing the following\n                     expression in <em class=\"strong\">3ds Max</em> R2.5 results in a value of <span class=\"code\"> <em class=\"strong\">Box</em> </span> because <span class=\"code\"> <em class=\"strong\">Box</em> </span> is a <em class=\"strong\">3ds Max</em> object class. In 3DS R3, MAXScript detects that the first use of variable <span class=\"code\"> <em class=\"strong\">box</em> </span> is an assignment and creates an implicitly declared local variable <span class=\"code\"> <em class=\"strong\">box</em> </span>. The result of this expression in <em class=\"strong\">3ds Max</em> R3 is the value <span class=\"code\"> <em class=\"strong\">undefined</em> </span>. \n                  </p> \n               </li>\n            </ul> \n            <div class=\"codeBlock\"><a name=\"WS1A9193826455F5FF29A9843712B1ADDAC1F-5598\"></a><pre>(if false do box=10; box)</pre></div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869D7\"></a>In the following script, an error was introduced by using undefined variable <span class=\"code\"> <em class=\"strong\">k</em> </span> in line 7. In the output, the error call stack trace-back shows the value for variable\n               <span class=\"code\"> <em class=\"strong\">b</em> </span> in function <span class=\"code\"> <em class=\"strong\">afunc</em> </span> and in the block-expression calling <span class=\"code\"> <em class=\"strong\">afunc</em> </span>. \n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E869D8\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E869D9\"></a><p class=\"table-body\"> <em class=\"strong\">SCRIPT</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-619F\"></a><pre>\nfn afunc =\n(\nlocal b\nb = box()\nfor i in 1 to 10 do\nfor j in 1 to 10 do\ninstance b pos:[i*20,j*30,30*sin(k*36)]\n)\n--\n(\nglobal c=10\nlocal b\nb=\"hello\"\nafunc()\n)\n</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E869EB\"></a><p class=\"table-body\"> <em class=\"strong\">OUTPUT:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-619E\"></a><pre>\nafunc()\n-- Error occurred in j loop\n-- Frame:\n-- k: undefined\n-- j: 1\n-- called in i loop\n-- Frame:\n-- i: 1\n-- called in afunc()\n-- Frame:\n-- b: $Box:Box102 @ [0.000000,0.000000,0.000000]\n-- called in &lt;block&gt;()\n-- Frame:\n-- b: \"hello\"\n-- No \"\"*\"\" function for undefined\nOK\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E869FF\"></a>If the same script is run with lines 3 and 12 removed, the following output is generated.\n               Because function <span class=\"code\"> <em class=\"strong\">afunc</em> </span> and the block-expression are in different MAXScript scope contexts, variable <span class=\"code\"> <em class=\"strong\">b</em> </span> is implicitly declared as local in each and contain different values. However, because\n               they were implicitly defined, they are not included in the error call stack trace-back.\n               \n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86A00\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86A01\"></a><p class=\"table-body\"> <em class=\"strong\">OUTPUT:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-619D\"></a><pre>\nafunc()\n-- Error occurred in j loop\n-- Frame:\n-- k: undefined\n-- j: 1\n-- called in i loop\n-- Frame:\n-- i: 1\n-- called in afunc()\n-- Frame:\n-- No \"\"*\"\" function for undefined\nOK\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86A12\"></a> <span class=\"char_link\"><a href=\"GUID-3E792B8C-AF02-4178-9651-EDDB1341B68D.htm\">Specifying Global Variables As Global Using ::</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86A13\"></a> <span class=\"char_link\"><a href=\"GUID-2739C3A0-16DB-4871-9F69-DCEC0862A186.htm\">Persistent Global Variables</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86A14\"></a> <span class=\"char_link\"><a href=\"GUID-5C8F4D60-69B0-4DC0-9641-E0F8C0CFCB84.htm\">Type-Free Variables</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86A15\"></a> <span class=\"char_link\"><a href=\"GUID-FAF8B2F4-3199-4757-A604-49E1F5BBEBE9.htm\">Reference Assignment</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86A16\"></a> <span class=\"char_link\"><a href=\"GUID-608825CD-D025-4054-91E2-2562A6AB31CF.htm\">Visibility of Locals, Functions, Structures and User-Interface Items in Rollout Code\n                              </a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86A17\"></a> <span class=\"char_link\"><a href=\"GUID-D7B7488F-BE23-4A1E-BA83-A48C21F6C73B.htm\">Accessing Locals and Other Items in a Rollout from External Code</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86A19\"></a> <span class=\"char_link\"><a href=\"GUID-DF09BF99-2106-408D-9965-60436B853D76.htm\">Memory Allocation and Garbage Collection</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86A1A\"></a> <span class=\"char_link\"><a href=\"GUID-CA7ABB4B-6CB6-47B7-903D-84D2F45D993D.htm\">Manual Garbage Collection</a></span> \n                     </p> \n                  </li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";