var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-08-28\">\n      <meta name=\"topicid\" content=\"GUID-B48DA525-F64B-480C-A6E1-3A05D09007F7\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"indexterm\" content=\"parameters - functions\">\n      <meta name=\"indexterm\" content=\"function\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Function Parameters</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-B48DA525-F64B-480C-A6E1-3A05D09007F7\"></a><div class=\"head\">\n            <h1>Function Parameters </h1>\n         </div>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E8737F\"></a> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E87380\"></a>  The parameters to a function are implicitly declared as local variables at the head\n               of the function body, and are initialized with the arguments given by the caller.\n               The parameters have a scope that extends to the end of the function body \n               		  <span class=\"code\"> <em class=\"strong\">&lt;expr&gt;</em> </span>. They can be assigned to and hidden by nested local variables with the same name\n               as can other local variables. \n               		\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E87383\"></a>Keyword parameters can be given default values in the function definition, implicitly\n               making them optional to the caller. If the caller does not supply one of the keyword\n               arguments, it is initialized to the default value upon function entry. The default\n               values are specified after the \n               		  <span class=\"code\"> <em class=\"strong\">\':\'</em> </span> (colon). \n               		\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E87384\"></a>If a keyword parameter is defined with no default value and is not supplied by the\n               caller, it is initialized to the special value \n               		  <span class=\"code\"> <em class=\"strong\">unsupplied</em> </span>. You can check for this value in the body of your function and handle missing required\n               keyword argument errors accordingly. \n               		\n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E87385\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E87386\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-609C\"></a><pre>\nfn foo a b: c:\"default c\"  =\n(\n	\n	if b == unsupplied then format \"Where\'s the b: argument??\"\n	else format \"a:% b:% c:%\" a b c\n)\n\nfoo \"A\" b:\"B\"\n--&gt; a:A b:B c:default c\nfoo \"A\" b:\"B\" c:\"C\"\n--&gt; a:A b:B c:C\nfoo \"A\" c:\"C\"\n--&gt; Where\'s the b: argument??\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p>For more information about keyword arguments and default values, see \n               		  <span class=\"char_link\"><a href=\"GUID-822DE27D-004E-4B19-ADA6-B0E980626462.htm\">Positional and Keyword Arguments</a></span>.\n               		\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E8738F\"></a>As described in \n               		  <span class=\"char_link\"><a href=\"GUID-FAF8B2F4-3199-4757-A604-49E1F5BBEBE9.htm\">Reference Assignment</a></span>, MAXScript uses an assignment called \n               		  <em class=\"mild\">reference assignment</em>. In a function, each parameter variable contains a reference to the value passed\n               to the function by the caller. Assigning to a parameter variable places a new reference\n               in the parameter variable, and has no effect on the variable values in the caller\n               (it is an entirely local action). An exception to this is when the parameter value\n               is a compound value such as, a 3D Point, string, or array, and you assign a new value\n               to one of the components of the compound value. In this case, the variable\'s reference\n               does not change, and the changed compound value is still referenced by the variable\n               in the caller. \n               		\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E87390\"></a>While in some cases you might want a function to return multiple values, it is typically\n               not a good practice to do this by assigning to the components of a parameter variable\n               compound value. To manipulate a compound value passed as a parameter, a compound value\n               needs to be created in the caller and passed to the function, which can give results\n               as shown in this example: \n               		\n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E87391\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E87392\"></a><p class=\"table-body\"> <em class=\"strong\">SCRIPT:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-609B\"></a><pre>\nfn getXYZset val =\n(\nval.x=random -100 100\nval.y=random 100 100\nval.z=random val.x val.y\n)\n(\nv1=[0,0,0]\nv4=v1\ngetXYZSet v1\nformat \"v1= %; v4= %\\n\" v1 v4\ngetXYZSet v4\nformat \"v1= %; v4= %\\n\"v1 v4\n)\n</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E873A3\"></a><p class=\"table-body\"> <em class=\"strong\">OUTPUT:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-609A\"></a><pre>\ngetXYZset() -- result lines 1 to 5\nv1= [-84,100,78.1224]; v4= [-84,100,78.1224] -- output line 10\nv1= [10,100,18.6575]; v4= [10,100,18.6575] -- output line 12\nOK -- result lines 6 to 12\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E873A9\"></a>Note that it appears the value for both variables \n               		  <span class=\"code\"> <em class=\"strong\">v1</em> </span> and \n               		  <span class=\"code\"> <em class=\"strong\">v4</em> </span> are set in the calls to \n               		  <span class=\"code\"> <em class=\"strong\">getXYZset</em> </span>, even though only one variable is being passed. The reason for this is in line 8,\n               variable \n               		  <span class=\"code\"> <em class=\"strong\">v4</em> </span> is initialized to the value of \n               		  <span class=\"code\"> <em class=\"strong\">v1</em> </span>. What actually occurs is that in line 7, a reference to a Point3 value of [0,0,0]\n               is created and this reference is stored in \n               		  <span class=\"code\"> <em class=\"strong\">v1</em> </span>. In line 8, that same reference is retrieved from \n               		  <span class=\"code\"> <em class=\"strong\">v1</em> </span> and assigned to \n               		  <span class=\"code\"> <em class=\"strong\">v4</em> </span>. Rather than storing two different values, both \n               		  <span class=\"code\"> <em class=\"strong\">v1</em> </span> and \n               		  <span class=\"code\"> <em class=\"strong\">v4</em> </span> reference the same value. When you set the components of the compound value in \n               		  <span class=\"code\"> <em class=\"strong\">getXYZset</em> </span>, the reference to the Point3 value is not being changed, and both variables still\n               reference that value. \n               		\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E873AA\"></a>The proper way to return multiple values from a function is to have the return value\n               of the function be an array or a structure. For example, \n               		\n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E873AB\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E873AC\"></a><p class=\"table-body\"> <em class=\"strong\">SCRIPT:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6099\"></a><pre>\nfn readDataFile filename =\n(\nf=openfile filename\ndata=#()\navg=0\nnVals=0\nwhile not eof f do\n(\nval = readvalue f\nappend data val\navg += val\nnVals += 1\n)\nclose f\n#(data,(avg/nVals))\n)\nresult=readDataFile \"c:\\\\datafiles\\\\run1.dat\"\ndata=result[1]\navg=result[2]\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E873C3\"></a>Normally, if you are writing a function that is operating on a compound value that\n               is passed as a parameter, you must make a \n               		  <span class=\"code\"> <em class=\"strong\">copy</em> </span> of the value. This prevents the value in the caller from inadvertently being changed.\n               For example, \n               		\n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E873C4\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E873C5\"></a><p class=\"table-body\"> <em class=\"strong\">SCRIPT:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6098\"></a><pre>\nfn uppercase instring =\n(\nlocal upper, lower, outstring\nupper=\"ABCDEFGHIJKLMNOPQRSTUVWXYZ\"\nlower=\"abcdefghijklmnopqrstuvwxyz\"\noutstring=copy instring -- operate on copy of instring\nfor i=1 to outstring.count do\n(\nj=findString lower outstring[i]\nif j != undefined do outstring[i]=upper[j]\n)\nreturn outstring\n)\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";