var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-08-28\">\n      <meta name=\"topicid\" content=\"GUID-CA82121D-8CA7-4739-A5E4-545534EF6C75\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"indexterm\" content=\"polymorphism\">\n      <meta name=\"indexterm\" content=\"inheritance\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Inheritance and Polymorphism</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-CA82121D-8CA7-4739-A5E4-545534EF6C75\"></a><div class=\"head\">\n            <h1>Inheritance and Polymorphism </h1>\n         </div>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4CFE\"></a>  Inheritance and polymorphism are the two defining characteristics of object-oriented\n            languages. <em class=\"mild\">Inheritance</em> is the ability of an object class to inherit the operations and properties of its\n            parent class. <em class=\"mild\">Polymorphism </em>is the ability of a single-named operation to work on different values of different\n            object classes. We\'ll explore them here to see how they are used in <em class=\"strong\">3ds Max</em> and MAXScript. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4D01\"></a>First and foremost, both characteristics depend on the system having a well-defined\n            type, or <em class=\"mild\">class</em>, for every value. Assigning classes to values divides all the values you can work\n            with into groups, each of which have well-defined operations and properties for their\n            values. In some languages, like standard C++, the class is only manifest at compile-time.\n            In others, like Smalltalk and MAXScript, the classes are actual runtime entities and\n            every value has a runtime class-tag that says what class it is. This is the main reason\n            MAXScript can have type-free variables. It can look at the class-tag of the value\n            in a variable at runtime to determine its class. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4D02\"></a>Once the objects are grouped into classes, the groups themselves can be arranged into\n            hierarchies. We do this all the time in real life. A classic example is the biological\n            hierarchy, arranging real-life objects hierarchically into animals and plants and\n            mammals and vertebrates and humans and adults and children, and so on. An object-oriented\n            language provides a way to arrange value classes into hierarchies and, like the hierarchies\n            we are used to, a class at some point in the hierarchy shares all the operations and\n            properties of all its parent classes. This sharing is known as <em class=\"mild\">inheritance </em>in object-oriented languages, and the operations and properties defined for a class\n            are automatically inherited by all of its descendant classes. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4D03\"></a>As an example, the following is the hierarchy for the Box class: \n         </p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4D04\"></a><div class=\"table-ruled\">\n            <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n               <colgroup>\n                  <col width=\"100%\">\n               </colgroup>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS1A9193826455F5FF-56C0CD8012AE38BF917-6386\"></a><pre>\nValue\n  MAXWrapper\n    Node\n      GeometryClass\n      &nbsp;&nbsp;Box\n</pre></div> \n                  </td>\n               </tr>\n            </table>\n         </div>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4D0D\"></a>Various characteristics are defined for a class, and each lower class inherits those\n            characteristics. For example, the <span class=\"code\"> <em class=\"strong\">classOf() </em> </span>method is defined for class Value, and class Box inherits this method through its\n            hierarchy. Thus you can specify a Box object as a parameter to the <span class=\"code\"> <em class=\"strong\">classOf() </em> </span>method. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4D0E\"></a>In the <span class=\"char_link\"><a href=\"\">MAXScript Class Hierarchy</a></span> topic, all the built-in classes in MAXScript are laid out in their place in the hierarchy\n            so you can see for each class the classes it inherits characteristics from. Some of\n            the hierarchy comes from within <em class=\"strong\">3ds Max </em>itself which is internally object-oriented. All of its objects, modifiers, controllers,\n            and so on are arranged in a well-defined inheritance hierarchy. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4D0F\"></a>Once the objects have been grouped into classes, the symbology for the operations\n            that can be performed on objects of each class can be simplified. The classic example\n            of this comes from mathematics. The symbolic operators \'+\', \'-\', \'*\', and \'/\' are\n            shared among many types of values (integers, reals, complex, vectors, matrices, and\n            so on), and perform the correct action on the types they are applied to. This capability\n            for a single-named operation to work on different types is called <em class=\"mild\">polymorphism</em>. Object-oriented languages let you use the same name for different methods or operators\n            across different classes, and the correct method to apply is automatically determined\n            based on the class of the value it is used with. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4D10\"></a>In MAXScript, all math operators and methods are polymorphic in this sense and reflect\n            the standard mathematical conventions. An interesting example for <em class=\"strong\">3ds Max </em>use is the <span class=\"code\"> <em class=\"strong\">random() </em> </span>method. It takes two arguments and generates a random value between them. So, if you\n            give it floats it gives you a float back; if you give it 3D points, it gives you a\n            random point in the box they define the corners of; if you give it quaternions, it\n            gives you back a random rotation between them, and so on. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC4D11\"></a>Further, all the methods that work on <em class=\"strong\">3ds Max</em> objects are polymorphic within their hierarchies. So move, hide, and select work\n            on all types of scene objects; time scaling and insertion work on all the types of\n            controllers, and so on. \n         </p>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";